/*== SpyCCode.c ========================================================================
  
   Vehicle Spy C-Code interface file
   
   -------------------------------------------------------------------------------------
   XCP Driver Implementation
   -------------------------------------------------------------------------------------
   Copyright (c) 2009 by Intrepid Control Systems
   All Rights Reserved.
   
   -------------------------------------------------------------------------------------
   $Log: SpyCCode.c,v $
   Revision 1.3  2011-04-01 20:04:47  coltc
   saftey updates

   Revision 1.2  2010-01-13 13:02:13  ericq
   changes made during eric instruction video

   Revision 1.2  2009-07-16 18:02:57  coltc
   Reomved authors and added calpage commands

   Revision 1.1  2009-07-02 22:20:29  coltc
   First XCP Driver commit !!!

   Initial revision
  ======================================================================================
*/
#include "vspy.h"
#include "SpyCCode.h"
#include "ecu.h"

/*======================================================================================
// GLOBAL VARIABLES: 
// -------------------------------------------------------------------------------------
// These global variables are used to implement the XCP protocol
//====================================================================================*/

/* -------------------------------------------------------------------------------------
// All XCP data
*/
XCPProtocol tXCP;
/* -----------------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------------------
// Command received data
*/
CANData* pRecData;
/* -----------------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------------------
// Command transmit data - 
// [ECU CODE CHANGE NOTE]
// The structure TX_ECU_To_Tester_HS_CAN is generated by
// the Vehicle Spy C-Code interface.  Replace "tECUtoTester.MessageData.btData" with 
// the start address of your maximum 8 Bytes of CAN data for the real ECU.
*/
TX_ECU_To_Tester_HS_CAN tECUtoTester;
CANData* pSendData = (CANData*)tECUtoTester.MessageData.btData;
/* -----------------------------------------------------------------------------------*/


/*======================================================================================
// FUNCTION: SysMemSetMM
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - Remove the code inside #ifdef ECU_MEMORY_SIMULATION for a 
// real ECU.  This code is designed to set memory from the dummy ECU memory in 
// this simulation of an XCP ECU.
//====================================================================================*/
void SysMemSet( _u08* pStart, _u16 cnt, _u08 value )
{
	for( ; cnt > 0; cnt-- )
	{
		*pStart = value;
		pStart++;
	}	
}                


/*======================================================================================
// FUNCTION: SysMemClrMM
// -------------------------------------------------------------------------------------
// SysMemClr - Set num bytes to 0 at pStart
//====================================================================================*/
void SysMemClr( _u08* pStart, _u16 cnt )
{
	SysMemSet( pStart, cnt, 0 );
}


/*======================================================================================
// FUNCTION: SysCopyMemory
// -------------------------------------------------------------------------------------
// Copies ECU memory from pSrc to pDst.  
//====================================================================================*/
void SysMemCpy( _u08* pDst, const _u08* pSrc, _u16 cnt )
{
	for ( ; cnt > 0; cnt-- )
	{
		*pDst = *pSrc;
		pDst++;
		pSrc++;
	}	
}

/*======================================================================================
// FUNCTION: WriteMTA
// -------------------------------------------------------------------------------------
// Write num bytes of data at *data to the current MTA pointer
//====================================================================================*/
void WriteMTA( const _u08* data, _u08 cnt )
{
	SysMemCpy( tXCP.pMTA, data, cnt );
	(_u08)tXCP.pMTA =+ cnt;
}


/*======================================================================================
// FUNCTION: ReadMTA
// -------------------------------------------------------------------------------------
// Read num bytes of data at *data to the current MTA pointer
//====================================================================================*/
void ReadMTA( _u08* data, _u08 cnt )
{
	SysMemCpy( data, tXCP.pMTA, cnt );
	(_u08)tXCP.pMTA =+ cnt;
}


/*======================================================================================
// FUNCTION: InitDAQLists
// -------------------------------------------------------------------------------------
// Initalize the DAQ Lists to a known state
// [ECU CODE CHANGE NOTE] - This function should be called on power up of the ECU
//====================================================================================*/
_u08 InitDAQLists()
{
	/* Clear the entire XCP structure */
	SysMemClr( (_u08*)&tXCP, sizeof(XCPProtocol) );

	// INIT DAQ List 1
	tXCP.tDAQLists[0].pODTs = tXCP.tODTDAQList1;
	tXCP.tDAQLists[0].ucFirstODT = 0;
	tXCP.tDAQLists[0].ucNumODTs = NUM_ODTS_DAQ_LIST_1;

	/* INIT DAQ List 2 */
	tXCP.tDAQLists[1].pODTs = tXCP.tODTDAQList2;
	tXCP.tDAQLists[1].ucFirstODT = NUM_ODTS_DAQ_LIST_1;
	tXCP.tDAQLists[1].ucNumODTs = NUM_ODTS_DAQ_LIST_2;

	/* INIT DAQ List 3 */
	tXCP.tDAQLists[2].pODTs = tXCP.tODTDAQList3;
	tXCP.tDAQLists[2].ucFirstODT = NUM_ODTS_DAQ_LIST_2+NUM_ODTS_DAQ_LIST_1;
	tXCP.tDAQLists[2].ucNumODTs = NUM_ODTS_DAQ_LIST_3;

	/* [ECU CODE CHANGE NOTE] if more DAQ Lists are added to the ECU project, more lists
	// need to be initalized here.
	*/

	return 0;
}


/*======================================================================================
// FUNCTION: StartDAQList
// -------------------------------------------------------------------------------------
// Start all selected DAQ lists
//====================================================================================*/
void StartDAQList( _u16 indx )
{
	tXCP.tDAQLists[indx].tStatus.bActive = 1;
	tXCP.ucSessionStatus |= (_u08)SS_DAQ;
}


/*======================================================================================
// FUNCTION: StartAllSelectedDAQLists
// -------------------------------------------------------------------------------------
// Start all selected DAQ lists
//====================================================================================*/
void StartAllSelectedDAQLists( void )
{
	_u08 indx;

	/* Loop and set bit to start DAQ'ing on all selected daq lists */
	for( indx = 0; indx < NUM_DAQ_LISTS; indx++ )
	{
		if( tXCP.tDAQLists[indx].tStatus.bSelected )
		{
			/* De-select this DAQ List */
			tXCP.tDAQLists[indx].tStatus.bSelected = 0;
			StartDAQList( indx );
		}
	}
}


/*======================================================================================
// FUNCTION: StopDAQList
// -------------------------------------------------------------------------------------
// Stop the DAQ List at indx
//====================================================================================*/
void StopDAQList( _u16 indx )
{
	tXCP.tDAQLists[indx].tStatus.bSelected = 0;
	tXCP.tDAQLists[indx].tStatus.bActive = 0;

	for( indx = 0; indx < NUM_DAQ_LISTS; indx++ )
	{
		if( tXCP.tDAQLists[indx].tStatus.bActive )
		{
			return;
		}
	}

	/* Reset the session status to not daqing if no DAQ lists running */
	tXCP.ucSessionStatus &= (_u08)((~SS_DAQ) & 0x00FF);
}


/*======================================================================================
// FUNCTION: StopAllSelectedDAQLists
// -------------------------------------------------------------------------------------
// Stop all selected DAQ Lists
//====================================================================================*/
void StopAllSelectedDAQLists( void )
{
	_u08 indx;

	for( indx = 0; indx < NUM_DAQ_LISTS; indx++ )
	{
		if( tXCP.tDAQLists[indx].tStatus.bSelected )
		{
			tXCP.tDAQLists[indx].tStatus.bSelected = 0;
			StopDAQList( indx );
		}
	}
}


/*======================================================================================
// FUNCTION: StopAllDAQLists
// -------------------------------------------------------------------------------------
// Stop all DAQ Lists
//====================================================================================*/
void StopAllDAQLists()
{
	_u08 indx;

	for( indx = 0; indx < NUM_DAQ_LISTS; indx++)
	{
		tXCP.tDAQLists[indx].tStatus.bActive = 0;
		tXCP.tDAQLists[indx].tStatus.bSelected = 0;
	}

	/* Reset the session status to not daqing if no DAQ lists running */
	tXCP.ucSessionStatus &= (_u08)(~SS_DAQ & 0x00FF);
}


/*======================================================================================
// FUNCTION: TranslateECUAddress
// -------------------------------------------------------------------------------------
// Translates addrsses from the A2L file into addresses.
// [ECU CODE CHANGE NOTE] - This function translates the address in the A2L file into
// an address for this simulation. This function can be removed for a real ECU.
//====================================================================================*/
_u08* TranslateECUAddress( _u32 ECUAddress )
{
	_u32 RelAddress = ECUAddress - SIMULATION_MEMORY_START_ADDRESS + 4;

	if( ECUAddress < SIMULATION_MEMORY_START_ADDRESS ) return 0;
	if( RelAddress > SIMULATION_MEMORY_SIZE ) return 0;

	return (_u08*)(RelAddress + tECU.START_ECU_MEMORY);
}


/*======================================================================================
// FUNCTION: FireEvent
// -------------------------------------------------------------------------------------
// Sets the DAQ event  Event and transmits the correct data on the can network.
//====================================================================================*/
void FireEvent( unsigned int Event )
{    
	TX_ECU_To_Tester_HS_CAN DTOPacket;
	_u08*	pData;  
	_u08*  pECUAddress;
	_u16	daqindx;
	_u08	odt;
	_u08	i;
	_u08	j;

	/* Data Transmit Object structure */
	static _u08 DataOut[8];

	/* If we are not DAQing then noop */
	if( !(tXCP.ucSessionStatus & SS_DAQ) )
		return;

	for( daqindx = 0; daqindx < NUM_DAQ_LISTS; daqindx++ )
	{
		/* If DAQ list is not running, skip it */
		if( !tXCP.tDAQLists[daqindx].tStatus.bActive )
			continue;
		
		/* Make sure the list is configured for this event */
		if( tXCP.tDAQLists[daqindx].ucEvent != Event )
			continue;

		/* For each ODT get its entries */
		odt = tXCP.tDAQLists[daqindx].ucFirstODT;
		for( i = 0; i < tXCP.tDAQLists[daqindx].ucLastUsedODT; i++)
		{
			DataOut[0] = odt;

			pData = (_u08*)&DataOut[1];
			for( j = 0; j < tXCP.tDAQLists[daqindx].pODTs[i].ucLastUsedODTEntry; j++ )
			{				
				/* ODT Entry size is alway 1 byte, for DAQ Items larger than 1 byte
				// they are split up into mutiple bytes. */
				pECUAddress = TranslateECUAddress( tXCP.tDAQLists[daqindx].pODTs[i].ulAddress[j] );
				if( pECUAddress )
				{
					SysMemCpy( pData, pECUAddress, 1 );
					pData++;
				}
			}
							
			/* Send the DAQ data

			// [ECU CODE CHANGE NOTE] - The TX_ECU_To_Tester_HS_CAN structure is a Vehicle Spy
			// specific structure used for transmiting data.  In an ECU project, replace this
			// structure with one CAN Frame structure for sending CAN data on the network.
			// btData holds the data for a CAN frame (max 8 bytes).*/
			TX_ECU_To_Tester_HS_CAN_Init(&DTOPacket);

			SysMemClr( DTOPacket.MessageData.btData, 8 );
			DTOPacket.MessageData.iNumDataBytes = (_u08)(pData - (&DataOut[0]));
			SysMemCpy( DTOPacket.MessageData.btData, (&DataOut[0]), 8 );
			
			TX_ECU_To_Tester_HS_CAN_Transmit( &DTOPacket );
		} 
	}
}


/*======================================================================================
// FUNCTION: Spy_Main
// -------------------------------------------------------------------------------------
// Main loop for the ECU simulation.
// [ECU CODE CHANGE NOTE] - This fucntion can be removed in a real ECU implementation.
//====================================================================================*/
void Spy_Main()
{
	int i = 0;
	int j = 0;

	/* Init DAQ memory var's */
	tXCP.ucSessionStatus = InitDAQLists();

	/* ECU simulation */
	/* The ECU structure can be removed for the Real ECU implementation */
	InitECU();

	/* Run loop */
	while( 1 ) 
	{
		Sleep( 10 );

		Fire010MSEventECU();
		FireEvent( 0 );

		if( !(i++ % 10) )
		{
			Fire100MSEventECU();
			FireEvent( 1 );
		}
	
		if( !(i % 50) ) 
		{
			Fire500MSEventECU();	
			FireEvent( 2 );
		}
	}
}


/*======================================================================================
// FUNCTION: SendXCPResponse
// -------------------------------------------------------------------------------------
// This function will send a XCP response to the tester tool.  This function uses the 
// data structure of Vehicle Spy. 
// [ECU CODE CHANGE NOTE] - For a real ECU implementation, replace this function with a
// function that sends the data contained in the "SendData" data structure.
//====================================================================================*/
void SendXCPResponse()
{
	tECUtoTester.MessageData.iNumDataBytes = 8;
	TX_ECU_To_Tester_HS_CAN_Transmit_raw( &tECUtoTester );
}


/*======================================================================================
// FUNCTION: HandleSetDAQListModeMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleSetDAQListModeMessage()
{
	_u16 daqindx = pRecData->b16[1];
	_u16 eventnum = pRecData->b16[2];

    if( daqindx >= NUM_DAQ_LISTS )
	{
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
	}
	else
	{
		tXCP.tDAQLists[daqindx].ucEvent = (_u08)eventnum;
		pSendData->b08[0] = SUCCESS;
	}
		
	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleStartStopDAQMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleStartStopDAQMessage()
{
	_u16 daqindx = 0;
	_u08 mode = 0;

    daqindx = pRecData->b16[1]; 
	mode = pRecData->b08[1];

    if( daqindx >= NUM_DAQ_LISTS )
	{
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
	}
	else
	{
		if( (mode == 1) || (mode == 2) )
		{
			tXCP.tDAQLists[daqindx].tStatus.bSelected = 1;

			if( mode == 1 )
			{
				StartDAQList( daqindx );
			}

			pSendData->b08[1] = tXCP.tDAQLists[daqindx].ucFirstODT;
		}
		else
		{
			StopDAQList( daqindx );
		}

		pSendData->b08[0] = 0xFF; //Positive response
	}

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleSetDAQPtrMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleSetDAQPtrMessage()
{
	_u16 daqlist = pRecData->b16[1]; 
	_u08  odtlist = pRecData->b08[4];
	_u08  odtentry = pRecData->b08[5];
	
    if( (daqlist >= NUM_DAQ_LISTS) ||
		(odtlist >= tXCP.tDAQLists[daqlist].ucNumODTs) ||
		(odtentry >= 7) )
    {
	    pSendData->b08[0] = ERR_ERROR; 
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
    }
	else
	{
		tXCP.usDAQNum = daqlist;
		tXCP.ucODTNum = odtlist;
		tXCP.ucODTEntryNum = odtentry;

		pSendData->b08[0] = SUCCESS;
	}

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleWriteDAQMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleWriteDAQMessage()
{
	if( pRecData->b08[2] != 1 )
	{
		/* must have a size of 1 byte.  Multi-byte items are broken down into bytes*/
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
	}
	else
	{
		/* Set the address for this ODT */
		tXCP.tDAQLists[tXCP.usDAQNum].pODTs[tXCP.ucODTNum].ulAddress[tXCP.ucODTEntryNum] = pRecData->b32[1];
		if(	tXCP.tDAQLists[tXCP.usDAQNum].ucLastUsedODT <= tXCP.ucODTNum )
		{
			tXCP.tDAQLists[tXCP.usDAQNum].ucLastUsedODT = tXCP.ucODTNum + 1;
		}
		if( tXCP.tDAQLists[tXCP.usDAQNum].pODTs[tXCP.ucODTNum].ucLastUsedODTEntry <= tXCP.ucODTEntryNum )
		{
			tXCP.tDAQLists[tXCP.usDAQNum].pODTs[tXCP.ucODTNum].ucLastUsedODTEntry = tXCP.ucODTEntryNum + 1;
		}

		pSendData->b08[0] = SUCCESS;
	}

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleClearDAQListMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleClearDAQListMessage()
{
	int daqindx = pRecData->b16[1];

    if( daqindx >= NUM_DAQ_LISTS )
    {
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
    }
    else
    {
		StopDAQList( daqindx );
		tXCP.tDAQLists[daqindx].ucEvent = 0;
		SysMemClr( (_u08*)tXCP.tDAQLists[daqindx].pODTs, sizeof(ODT)*tXCP.tDAQLists[daqindx].ucNumODTs );
		tXCP.tDAQLists[daqindx].ucLastUsedODT = 0;

		pSendData->b08[0] = SUCCESS;
    }

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleStartStopSynchMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleStartStopSynchMessage()
{
	_u08 mode = pRecData->b08[1];

    if( mode == 0 )
		StopAllDAQLists();
	else if( mode == 1 )
		StartAllSelectedDAQLists();
	else if( mode == 2 )
		StopAllSelectedDAQLists();

    pSendData->b08[0] = SUCCESS;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleGetDAQListInfoMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleGetDAQListInfoMessage()
{
	int daqlist = pRecData->b16[1];

	if( daqlist >= NUM_DAQ_LISTS )
    {
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
    }
    else
    {
		pSendData->b08[0] = SUCCESS;
        pSendData->b08[1] = DAQ_LIST_PREDEFINED | DAQ_LIST_DAQ; /* Only DAQ, no STIM, PREDEFINED "STATIC" */
        pSendData->b08[2] = tXCP.tDAQLists[daqlist].ucNumODTs;
        pSendData->b08[3] = 0x07; /* 7 entries per ODT */
		pSendData->b16[2] = tXCP.tDAQLists[daqlist].ucEvent;
    }

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleConnectMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleConnectMessage()
{
	tXCP.ucSessionStatus = SS_CONNECTED;

	pSendData->b08[0] = SUCCESS;
    pSendData->b08[1] = CONNECT_RESOURCE_DAQ|CONNECT_RESOURCE_CAL_PAG;	/* Resource */
    pSendData->b08[2] = 0x00;	/* Comm Mode Basic */
    pSendData->b08[3] = 0x08;	/* Max CTO size in bytes */
	pSendData->b16[2] = 0x08;	/* Max DTO size (word) */
    pSendData->b08[6] = 0x01;	/* XCP Protocol layer version */
    pSendData->b08[7] = 0x01;	/* XCP Transport layer version */

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleDisConnectMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleDisConnectMessage()
{
	StopAllDAQLists();
	InitDAQLists();
	tXCP.ucSessionStatus = 0;
}


/*======================================================================================
// FUNCTION: HandleSynchMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleSynchMessage()
{
    pSendData->b08[0] = ERR_ERROR;	/* This is a Positive response for the SYNCH command */
    pSendData->b08[1] = ERR_CMD_SYNCH;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleSetMTAMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleSetMTAMessage()
{
	tXCP.pMTA = TranslateECUAddress( pRecData->b32[1] );
	if( tXCP.pMTA )
	{
		pSendData->b08[0] = SUCCESS;
	}
	else
	{
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
	}

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleDownloadMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleDownloadMessage()
{
	WriteMTA( &pRecData->b08[2], pRecData->b08[1] );
    
	pSendData->b08[0] = SUCCESS;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleShortUploadMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleShortUploadMessage()
{
	tXCP.pMTA = TranslateECUAddress( pRecData->b32[1] );
	if( tXCP.pMTA )
	{
		ReadMTA( &pSendData->b08[1], pRecData->b08[1] );
		pSendData->b08[0] = SUCCESS;
	}
	else
	{
		pSendData->b08[0] = ERR_ERROR;
		pSendData->b08[1] = ERR_OUT_OF_RANGE;
	}

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleGetStatusMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleGetStatusMessage()
{
    pSendData->b08[0] = SUCCESS;

	if( tXCP.ucSessionStatus & SS_DAQ )
	{
		pSendData->b08[1] = GET_STATUS_DAQ_RUNNING;    
	}
	else
	{
		pSendData->b08[1] = 0x00;    
	}
	pSendData->b08[2] = 0x00; /* Resource protection, no protection by default */
	pSendData->b08[3] = 0x00; /* Reserved */
    pSendData->b16[2] = 0x00; /* Session configuration ID */

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleGetSeedMessage
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function returns a positve response with a dummy seed
// value.  
//====================================================================================*/
void HandleGetSeedMessage()
{
    pSendData->b08[0] = SUCCESS;
    pSendData->b08[1] = 0x04; /* Length of seed */
    pSendData->b08[2] = 0x00; /* Seed */
    pSendData->b08[3] = 0x00;
    pSendData->b08[4] = 0x00;
    pSendData->b08[5] = 0x00;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleUnlockMessage
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function returns a positve response with a dummy seed
// value.  
//====================================================================================*/
void HandleUnlockMessage()
{
    pSendData->b08[0] = SUCCESS;
    pSendData->b08[1] = 0x00; /* Resource state, everything unlocked */

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleSetCALPageMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleSetCALPageMessage()
{
	_u08 ucRequestedMode = pRecData->b08[1];
	_u08 ucRequestedLogicalDataSegmentNumber = pRecData->b08[2];
	_u08 ucRequestedLogicalDataPageNumber = pRecData->b08[3];
	
	tXCP.ucLastRequestedLogicalDataPageNumber = ucRequestedLogicalDataPageNumber;

	pSendData->b08[0] = SUCCESS;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleGetCALPageMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleGetCALPageMessage()
{
	pSendData->b08[0] = SUCCESS;
	pSendData->b08[1] = 0;
	pSendData->b08[2] = 0;
	pSendData->b08[3] = tXCP.ucLastRequestedLogicalDataPageNumber;

	SendXCPResponse();
}


/*======================================================================================
// FUNCTION: HandleCopyCALPageMessage
// -------------------------------------------------------------------------------------
// 
//====================================================================================*/
void HandleCopyCALPageMessage()
{
	pSendData->b08[0] = SUCCESS;

	SendXCPResponse();
}

/*======================================================================================
// FUNCTION: HandleXCPMessage
// -------------------------------------------------------------------------------------
// This is the main entry point for handeling XCP messages.  
// [ECU CODE CHANGE NOTE] - You must call this function in the real ECU implementation 
// for every XCP message that is received from Vehcle Spy.
//====================================================================================*/
void HandleXCPMessage( CANData* pRecData )
{

	if( pRecData->b08[0] == XCP_CONNECT )
	{
		HandleConnectMessage();
	}
	else
	{
		if( (tXCP.ucSessionStatus & SS_CONNECTED) == 0 )
		{
			return; /* Do not return a response if not connected. */
		}
		else
		{
			switch( pRecData->b08[0] )
			{
				case XCP_DISCONNECT:
					HandleDisConnectMessage();
				break;
				case XCP_GET_STATUS:
					HandleGetStatusMessage();
				break;
				case XCP_SYNC:
					HandleSynchMessage();
				break;
				case XCP_GET_SEED:
					HandleGetSeedMessage();
				break;
				case XCP_UNLOCK:
					HandleUnlockMessage();
				break;
				case XCP_SET_MTA:
					HandleSetMTAMessage();
				break;
				case XCP_SHORT_UPLOAD:
					HandleShortUploadMessage();
				break;
				case XCP_DOWNLOAD:
					HandleDownloadMessage();
				break;
				case XCP_CLEAR_DAQ_LIST:
					HandleClearDAQListMessage();
				break;
				case XCP_SET_DAQ_PTR:
					HandleSetDAQPtrMessage();
				break;
				case XCP_WRITE_DAQ:
					HandleWriteDAQMessage();
				break;
				case XCP_SET_DAQ_LIST_MODE:
					HandleSetDAQListModeMessage();
				break;
				case XCP_START_STOP_DAQ_LIST:
					HandleStartStopDAQMessage();
				break;
				case XCP_START_STOP_SYNCH:
					HandleStartStopSynchMessage();
				break;
				case XCP_GET_DAQ_LIST_INFO:
					HandleGetDAQListInfoMessage();
				break;
				case XCP_SET_CAL_PAGE:
					HandleSetCALPageMessage();
				break;
				case XCP_GET_CAL_PAGE:
					HandleGetCALPageMessage();
				break;
				case XCP_COPY_CAL_PAGE:
					HandleCopyCALPageMessage();
				break;
			}
		}
	}
}


/*======================================================================================
// FUNCTION: SpyMsg_MG_Tester_to_ECU_HS_CAN
// -------------------------------------------------------------------------------------
// Event handler for CAN messages from the XCP tool (Vehicle Spy). 
// 
// [ECU CODE CHANGE NOTE] - This function is used to get CAN messages from Vehicle Spy.
// In the real ECU driver, this function can be removed.  The call to "HandleXCPMessage"
// must be called in the real ECU implementation.
//====================================================================================*/
void SpyMsg_MG_TesterToECU_HS_CAN(MG_TesterToECU_HS_CAN * pMG_TesterToECU_HS_CAN)
{
	TX_ECU_To_Tester_HS_CAN_Init( &tECUtoTester );
	SysMemClr( tECUtoTester.MessageData.btData, 8 );
	pRecData = (CANData*)pMG_TesterToECU_HS_CAN->MessageData.btData;

	Spy_Lock();

	HandleXCPMessage( pRecData );

	Spy_Unlock();
}

/*======================================================================================
// FUNCTION: SpyMsg_TX_ECU_To_Tester_HS_CAN
// -------------------------------------------------------------------------------------
//
//====================================================================================*/
void SpyMsg_TX_ECU_To_Tester_HS_CAN(TX_ECU_To_Tester_HS_CAN * pTX_ECU_To_Tester_HS_CAN)
{
   // TODO: Add Event Code
}

/*=====================================================================================
// FUNCTION: Spy_EveryMessage
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_EveryMessage(GenericMessage * p_Msg)
{
}


/*=====================================================================================
// FUNCTION: Spy_EveryLoop
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_EveryLoop(unsigned int uiCurrentTime)
{
}


/*=====================================================================================
// FUNCTION: Spy_ErrorState
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_ErrorState(int iNetwork, int iTxErrorCount, int iRxErrorCount, int iErrorBitfield)
{
}


/*=====================================================================================
// FUNCTION: Spy_ErrorFrame
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_ErrorFrame(int iNetwork, int iTxErrorCount, int iRxErrorCount, int iErrorBitfield)
{
}


/*=====================================================================================
// FUNCTION: Spy_Stopped
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_Stopped()
{
}


/*=====================================================================================
// FUNCTION: Spy_KeyPress
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_KeyPress(int iKey, int iCTRLPressed, int iALTPressed)
{
}


/*=====================================================================================
// FUNCTION: Spy_Started
// -------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE] - This function is needed only by Vehicle Spy.  Can be removed
// for as an ECU driver.
//====================================================================================*/
void Spy_Started()
{
}