/*== SpyCCode.H ========================================================================
  
   Vehicle Spy C-Code interface file
   
   -------------------------------------------------------------------------------------
   XCP Driver Implementation
   -------------------------------------------------------------------------------------

   Copyright (c) 2009 by Intrepid Control Systems
   All Rights Reserved.
   
   -------------------------------------------------------------------------------------
   $Log: SpyCCode.h,v $
   Revision 1.3  2011-04-01 20:04:47  coltc
   saftey updates

   Revision 1.2  2010-01-13 13:02:13  ericq
   changes made during eric instruction video

   Revision 1.2  2009-07-16 18:02:57  coltc
   Reomved authors and added calpage commands

   Revision 1.1  2009-07-02 22:20:29  coltc
   First XCP Driver commit !!!

   Initial revision
  ======================================================================================
*/
#ifndef SPYCCODE_HEADER
#define SPYCCODE_HEADER

#define DAQLISTMEM	512


/*======================================================================================
// Type def defines
//====================================================================================*/
#define _u08 unsigned char
#define _s08 char
#define _u16 unsigned short
#define _s16 short 
#define _u32 unsigned long
#define _s32 long


/*======================================================================================
// Session Status defines
//====================================================================================*/
#define SS_ERROR							0x10 
#define SS_CONNECTED						0x20
#define SS_DAQ								0x40


/*======================================================================================
// Bits for GET_DAQ_LIST_INFO command
//====================================================================================*/
#define DAQ_LIST_PREDEFINED					0x01
#define	DAQ_LIST_EVENT_FIXED				0x02
#define DAQ_LIST_DAQ						0x04
#define DAQ_LIST_STIM						0x08


/*======================================================================================
// Bits for CONNECT command 
//====================================================================================*/
#define CONNECT_RESOURCE_CAL_PAG			0x01
#define	CONNECT_RESOURCE_DAQ				0x04
#define CONNECT_RESOURCE_STIM				0x08
#define CONNECT_RESOURCE_PGM				0x10
#define CONNECT_COMM_MODE_BYTE_ORDER		0x01
#define CONNECT_COMM_MODE_ADDRESS_GRAN1		0x02
#define CONNECT_COMM_MODE_ADDRESS_GRAN2		0x04


/*======================================================================================
// Bits for GET_STATUS command 
//====================================================================================*/
#define GET_STATUS_DAQ_RUNNING				0x40


/*======================================================================================
// Standard Error Codes
//====================================================================================*/
#define SUCCESS								0xFF
#define ERR_ERROR							0xFE
#define ERR_CMD_UNKNOWN						0x20
#define ERR_OUT_OF_RANGE					0x22
#define ERR_CMD_SYNCH						0x00


/*======================================================================================
// DAQ List defines
//-------------------------------------------------------------------------------------
// [ECU CODE CHANGE NOTE]
// Change these defines for differnt ECU projects.  Now there are 3 lists.  List 1 has
// 16 ODT's, List 2 has 32, List 3 has 64.  
// Generally NUM_ODT_ENTRY does not need to change as this is the max size of an ODT on
// CAN (7 bytes).
//====================================================================================*/
#define NUM_DAQ_LISTS						3			// Number of DAQ lists in ECU
#define NUM_ODTS_DAQ_LIST_1					16			// Number of ODT's DAQ List 1
#define NUM_ODTS_DAQ_LIST_2					32			// Number of ODT's DAQ List 2
#define NUM_ODTS_DAQ_LIST_3					64			// Number of ODT's DAQ List 3
#define NUM_ODT_ENTRY						7			// Number of ODT entries per ODT


/*======================================================================================
// Mandatory Standard Commands (STD)
//====================================================================================*/
#define XCP_CONNECT							0xFF		// IMPLEMENTED
#define XCP_DISCONNECT						0xFE		// IMPLEMENTED
#define XCP_GET_STATUS						0xFD		// IMPLEMENTED
#define XCP_SYNC							0xFC		// IMPLEMENTED


/*======================================================================================
// Optional Standard Commands (STD)
//====================================================================================*/
#define XCP_GET_COMM_MODE_INFO				0xFB		// NOT IMPLEMENTED
#define XCP_GET_ID							0xFA		// NOT IMPLEMENTED
#define XCP_SET_REQUEST						0xF9		// NOT IMPLEMENTED
#define XCP_GET_SEED						0xF8		// IMPLEMENTED
#define XCP_UNLOCK							0xF7		// IMPLEMENTED
#define XCP_SET_MTA							0xF6		// IMPLEMENTED
#define XCP_UPLOAD							0xF5		// NOT IMPLEMENTED
#define XCP_SHORT_UPLOAD					0xF4		// IMPLEMENTED
#define XCP_BUILD_CHECKSUM					0xF3		// NOT IMPLEMENTED                                         
#define XCP_TRANSPORT_LAYER_CMD				0xF2		// NOT IMPLEMENTED
#define XCP_USER_CMD						0xF1		// NOT IMPLEMENTED


/*======================================================================================
// Calibration Commands (mandatory) (CAL)
//====================================================================================*/
#define XCP_DOWNLOAD						0xF0		// IMPLEMENTED


/*======================================================================================
// Calibration Commands (optional) (CAL)
//====================================================================================*/
#define XCP_DOWNLOAD_NEXT					0xEF		// NOT IMPLEMENTED
#define XCP_DOWNLOAD_MAX					0xEE		// NOT IMPLEMENTED
#define XCP_SHORT_DOWNLOAD					0xED		// NOT IMPLEMENTED
#define XCP_MODIFY_BITS						0xEC		// NOT IMPLEMENTED


/*======================================================================================
// Page Switching Commands (optional) (PAG)
//====================================================================================*/
#define XCP_SET_CAL_PAGE					0xEB		// Function is stubbed, always returns true
#define XCP_GET_CAL_PAGE					0xEA		// Function is stubbed, always returns true
#define XCP_GET_PAG_PROCESSOR_INFO			0xE9		// NOT IMPLEMENTED
#define XCP_GET_SEGMENT_INFO				0xE8		// NOT IMPLEMENTED
#define XCP_GET_PAGE_INFO					0xE7		// NOT IMPLEMENTED
#define XCP_SET_SEGMENT_MODE				0xE6		// NOT IMPLEMENTED
#define XCP_GET_SEGMENT_MODE				0xE5		// NOT IMPLEMENTED
#define XCP_COPY_CAL_PAGE					0xE4		// Function is stubbed, always returns true


/*=====================================================================================
// Data Acquisition and Stimulation Commands (mandatory) (DAQ)
=====================================================================================*/
#define XCP_CLEAR_DAQ_LIST					0xE3		// IMPLEMENTED
#define XCP_SET_DAQ_PTR						0xE2		// IMPLEMENTED
#define XCP_WRITE_DAQ						0xE1		// IMPLEMENTED
#define XCP_SET_DAQ_LIST_MODE				0xE0		// IMPLEMENTED
#define XCP_GET_DAQ_LIST_MODE				0xDF		// IMPLEMENTED
#define XCP_START_STOP_DAQ_LIST				0xDE		// IMPLEMENTED
#define XCP_START_STOP_SYNCH				0xDD		// IMPLEMENTED


/*=====================================================================================
// Data Acquisition and Stimulation Commands (optional) (DAQ)
//===================================================================================*/
#define XCP_GET_DAQ_CLOCK					0xDC		// NOT IMPLEMENTED
#define XCP_READ_DAQ						0xDB		// NOT IMPLEMENTED
#define XCP_GET_DAQ_PROCESSOR_INFO			0xDA		// NOT IMPLEMENTED
#define XCP_GET_DAQ_RESOLUTION_INFO			0xD9		// NOT IMPLEMENTED 
#define XCP_GET_DAQ_LIST_INFO				0xD8		// NOT IMPLEMENTED
#define XCP_GET_DAQ_EVENT_INFO				0xD7		// NOT IMPLEMENTED
#define XCP_FREE_DAQ						0xD6		// NOT IMPLEMENTED
#define XCP_ALLOC_DAQ						0xD5		// NOT IMPLEMENTED
#define XCP_ALLOC_ODT						0xD4		// NOT IMPLEMENTED
#define XCP_ALLOC_ODT_ENTRY					0xD3		// NOT IMPLEMENTED


/*=====================================================================================
// Non-volatile memory programming commands (mandatory) (PGM)
//===================================================================================*/
#define XCP_PROGRAM_START					0xD2		// NOT IMPLEMENTED
#define XCP_PROGRAM_CLEAR					0xD1		// NOT IMPLEMENTED
#define XCP_PROGRAM							0xD0		// NOT IMPLEMENTED
#define XCP_PROGRAM_RESET					0xCF		// NOT IMPLEMENTED


/*=====================================================================================
// Non-volatile memory programming commands (optional) (PGM)
//===================================================================================*/
#define XCP_GET_PGM_PROCESSOR_INFO			0xCE		// NOT IMPLEMENTED
#define XCP_GET_SECTOR_INFO					0xCD		// NOT IMPLEMENTED
#define XCP_PROGRAM_PREPARE					0xCC		// NOT IMPLEMENTED
#define XCP_PROGRAM_FORMAT					0xCB		// NOT IMPLEMENTED 
#define XCP_PROGRAM_NEXT					0xCA		// NOT IMPLEMENTED
#define XCP_PROGRAM_MAX						0xC9		// NOT IMPLEMENTED
#define XCP_PROGRAM_VERIFY					0xC8		// NOT IMPLEMENTED


/*=====================================================================================
// UNION: CANData
// -----------------------------------------------------------------------------------
// union for accessing b08, b16, or b32 bountries, size is 8 bytes for size of single
// CAN frame.
//===================================================================================*/
typedef union 
{
	_u08	b08[8];		//  8-BIT access
	_u16	b16[4];		// 16-BIT access
	_u32	b32[2];		// 32-BIT access
} CANData;


/*=====================================================================================
// STRUCT: DAQStatus
// ------------------------------------------------------------------------------------
// Structure for DAQ List status
//===================================================================================*/
typedef struct
{
	_u08 bSelected  : 1;	// True if this list is selected
	_u08 bActive	: 1;	// True if this list is selected
} DAQListStatus;


/*======================================================================================
// STRUCT: ODT
// -------------------------------------------------------------------------------------
// Holds ODT's for a DAQ List
// Size of each ODT Entry is assumed to be 1 always.  Therefore each ODT will have 7
// entries for XCP on CAN.  If a Measurement Item in the A2L file is larger than a size
// of 1, it will be broken up into mutiple items of size one (into bytes)
//===================================================================================*/
typedef struct
{
	_u32 ulAddress[7];			// Max ODT Entries is 7 for CAN
	_u08 ucLastUsedODTEntry;	// Last used ODT Entry for this ODT
} ODT;


/*=====================================================================================
// STRUCT: DAQList
// -------------------------------------------------------------------------------------
// Structure for holding DAQ List information
//===================================================================================*/
typedef struct
{
	_u08	ucEvent;			// Event number of this DAQ List
	_u08	ucFirstODT;			// Absolute index of First ODT for this DAQ List
	_u08	ucNumODTs;			// Number of ODTs for this DAQ List, set in "InitDAQ"
	_u08    ucLastUsedODT;		// Last used ODT for this list
	DAQListStatus	tStatus;			// Status flags of the DAQ List
	ODT*			pODTs;				// Pointer to a list of ODT's, number of ODT's defined in XCPStruct	
} DaqList;


/*=====================================================================================
// STRUCT: XCPProtocol
// -------------------------------------------------------------------------------------
// Structure for holding all XCP information
//===================================================================================*/
typedef struct 
{
	/* Session Status of this ECU */
	_u08	ucSessionStatus;

	/* Memory transfer address */
	_u08*	pMTA;

	/* Last requested logical page number */
	_u08	ucLastRequestedLogicalDataPageNumber;

	/* Selected DAQ_LIST_NUMBER, ODT_NUMBER, ODT_ENTRY_NUMBER for the SET_DAQ_PTR command */
	_u16	usDAQNum;
	_u08	ucODTNum;
	_u08	ucODTEntryNum;

	// DAQ Lists
	DaqList	tDAQLists[NUM_DAQ_LISTS];

	// ODT List for each DAQ List, "pODTs" member in DAQList set to tODTDAQListX in "DAQInit"
	ODT tODTDAQList1[NUM_ODTS_DAQ_LIST_1];		// ODTs for DAQ List 1
	ODT tODTDAQList2[NUM_ODTS_DAQ_LIST_2];		// ODTs for DAQ List 2
	ODT tODTDAQList3[NUM_ODTS_DAQ_LIST_3];		// ODTs for DAQ List 3
	// [ECU CODE CHANGE NOTE] Add more tODTDAQListsX for more DAQ Lists in your ECU

} XCPProtocol;


#endif